data {
  int<lower=0>       n_all ;  // number of observations
  int<lower=0>       n_item;  // number of participants (items)
  int<lower=0>       n_meth;  // number of methods
  real             y[n_all];  // outcome variable
  int<lower=1>  item[n_all];  // item index
  int<lower=1>  meth[n_all];  // method index
}

transformed data {
  int<lower=1>  n_factor;
  n_factor = n_all / n_meth;
}

parameters {
  // Parameters for each method.
  vector[n_meth]  alpha;  // mean effect
  vector[n_meth]  sigma;  // sd

  // Random effects due to item.
  vector[n_item]  u_raw;  // per-item random effect
  real<lower=0>   gamma;  // sd

  // Random effects due to interaction between items and methods.
  vector[n_item]  c_raw[n_meth];  // per-combination random effect
  real<lower=0>     tau        ;  // sd
}

model {
  vector[n_item]  u        ;  // per-item random effect
  vector[n_item]  c[n_meth];  // per-combination random effect

  // Prior for fixed components.
  alpha ~ normal(0, 10000);

  // Priors for variance components.
  gamma ~ cauchy(0, 2.5);
  sigma ~ cauchy(0, 2.5);
  tau   ~ cauchy(0, 2.5);

  // Distributions of random effects.
  u_raw ~ normal(0, 1);
  u = gamma * u_raw;

  for (i in 1:n_meth) {
    c_raw[i] ~ normal(0, 1);
    c[i] = tau * c_raw[i];
  }

  // Distribution of the outcome variable.
  for (n in 1:n_all) {
    y[n] ~ normal(alpha[meth[n]] +
                      u[item[n]] +
             c[meth[n], item[n]],
                  sigma[meth[n]]);
  }
}

generated quantities {
  real<lower=0> sigma_ratio[1];
  real alpha_diff[1];
  
  sigma_ratio[1] <- sigma[1]/sigma[2];
  alpha_diff[1]  <- alpha[1]-alpha[2];
}
