data {
  int<lower=0>           n_all ;  // number of observations
  int<lower=0>           n_item;  // number of participants (items)
  int<lower=0>           n_meth;  // number of methods
  real                 y[n_all];  // outcome variable
  int<lower=1>      item[n_all];  // item index
  int<lower=1>      meth[n_all];  // method index
  int<lower=1>  repl_ind[n_all];  // item-replicate index
}

transformed data {
  int<lower=1>  n_factor;
  n_factor = n_all / n_meth;
}

parameters {
  // Parameters for each method.
  vector[n_meth]  alpha;  // mean effect
  real            sigma;  // sd

  // Random effects due to item.
  vector[n_item]  u_raw;  // per-item random effect
  real<lower=0>   gamma;  // sd

  // Random effects due to interaction between items and methods.
  vector[n_item]  c_raw[n_meth];  // per-combination random effect
  real<lower=0>     tau        ;  // sd

  // Random effects due to interaction between items and replicates.
  vector[n_factor]  a_raw;  // item-replicate interaction effect
  real<lower=0>     omega;  // sd
}

model {
  vector[n_item]    u        ;  // per-item random effect
  vector[n_item]    c[n_meth];  // per-combination random effect
  vector[n_factor]  a        ;  // item-replicate interaction effect

  // Prior for fixed components.
  alpha ~ normal(0, 10000);

  // Priors for variance components.
  gamma ~ cauchy(0, 2.5);
  sigma ~ cauchy(0, 2.5);
  tau   ~ cauchy(0, 2.5);
  omega ~ cauchy(0, 2.5);

  // Distributions of random effects.
  u_raw ~ normal(0, 1);
  u = gamma * u_raw;

  for (i in 1:n_meth) {
    c_raw[i] ~ normal(0, 1);
    c[i] = tau * c_raw[i];
  }

  for (i in 1:n_factor) {
    a_raw[i] ~ normal(0, 1);
    a[i] = omega * a_raw[i];
  }

  // Distribution of the outcome variable.
  for (n in 1:n_all) {
    y[n] ~ normal(alpha[meth[n]] +
                      u[item[n]] +
                  a[repl_ind[n]] +
             c[meth[n], item[n]],
                          sigma);
  }
}

generated quantities {
  real alpha_diff[1];
  alpha_diff[1]  <- alpha[1]-alpha[2];
}
