functions {
  /**
   * Evaluate the tracer gas quantity at a particular time point.
   * This is a two-component exponential decay model.
   *
   * @param t     Time point at which to evaluate the gas quantity
   * @param beta0 Mixture weight parameter
   * @param beta  Exponents for each component
   *
   * @return Gas quantity (between 0 and 1) at the given time point.
   */
  real gas_curve(real t, real beta0, vector beta) {
    return beta0 * exp(-beta[1] * t) + (1 - beta0) * exp(-beta[2] * t);
  }

  /**
   * Determine if the tracer gas quantity crosses a given threshold between
   * two time points, t1 and t2.
   *
   * @param t1        First time point (left end of desired time interval)
   * @param t2        Second time point (right end of desired time interval)
   * @param threshold Threshold for the gas quantity
   * @param beta0    Mixture weight parameter
   * @param beta     Exponents for the mixture components
   *
   * @return Boolean value: true if the threshold is crossed, false otherwise
   */
  int gas_passes_threshold(real t1, real t2, real threshold, real beta0,
                           vector beta) {
    return gas_curve(t1, beta0, beta) >  threshold &&
           gas_curve(t2, beta0, beta) <= threshold;
  }
}

data {
  int         M ;  // number of observations
  real    gas[M];  // tracer gas quantity (GAS)
  real  cevgm[M];  // cumulative expired volume of gas mixture (CEVGM)
  real  cevtg[M];  // cumulative expired volume of tracer gas  (CEVTG)
  int       k[M];  // breath index
}

transformed data {
  int       nmax       ;
  real  cevgm_tf[M - 1];
  real  cevtg_tf[M - 1];

  // Resolution for line search used to deterimine the real-valued stopping
  // 'breath'.
  nmax = 100;

  // Transform cumulative data to log increment scale.
  for (m in 1:(M - 1)) {
    cevgm_tf[m] = log(cevgm[m + 1] - cevgm[m]);
    cevtg_tf[m] = log(cevtg[m + 1] - cevtg[m]);
  }
}

parameters {
  // GAS curve.
  real<lower=0, upper=1>  beta0;  
  positive_ordered[2]      beta; // a.k.a 'beta1' and 'beta2' (log scale)
  real<lower=0>       error_gas;

  // CEVGM curve.
  real<lower=0>        beta5;
  real<lower=0>  error_cevgm;

  // CEVTG curve.
  real<lower=0>        beta3;
  real<lower=0>        beta4;
  real<lower=0>  error_cevtg;
}

transformed parameters {
  // Stopping breaths.
  real<lower=0>  m_stop_40;
  real<lower=0>  m_stop_40_cont;

  // MBW quantities using standard stopping breath.
  real<lower=0>  lci40;
  real<lower=0>  cev40;

  // MBW quantities using model-based stopping breath.
  real<lower=0>  lci40_cont;
  real<lower=0>  cev40_cont;

  // Define MBW quantities from model parameters.
  // Use a threshold of 1/40 (= 0.025) for the gas concentration.
  for (m in 2:100) {
    if (gas_passes_threshold(m - 1, m, 0.025, beta0, beta)) {
      // Use the standard stopping breath, k(40).
      m_stop_40 = m;
      cev40 = beta5 * m_stop_40;
      lci40 = cev40 / beta3;  // n.b. FRC = beta3

      // Use the model-based stopping 'breath', theta.
      // This stopping breath is real-valued rather than integer-valued.
      // It is determined by a simple line search.
      for (i in ((m - 1) * nmax):(m * nmax)) {
        if (gas_passes_threshold(1.0 / nmax * (i - 1),
                                 1.0 / nmax *  i,      0.025, beta0, beta)) {
          m_stop_40_cont = 1.0 / nmax * i;
          cev40_cont = beta5 * m_stop_40_cont;
          lci40_cont = cev40_cont / beta3;  // n.b. FRC = beta3
        }
      }
    }
  }
}

model {
  // Declare local variables.
  // These are used to store the mean and sd of the 3 curves, see below.
  vector[M]        gas_mean;
  vector[M - 1]  cevgm_mean;
  vector[M - 1]  cevtg_mean;
  vector[M]        gas_sd;
  vector[M - 1]  cevgm_sd;
  vector[M - 1]  cevtg_sd;


  // Priors (diffuse).

  beta0   ~ beta(2, 2);  // a.k.a. beta0
  beta[1] ~ normal(0, 1);
  beta[2] ~ normal(0, 1);

  beta3 ~ normal(0, 1000);
  beta4 ~ normal(0, 1);

  beta5 ~ normal(0, 100);

  error_gas   ~ cauchy(0, 2.5);
  error_cevgm ~ cauchy(0, 2.5);
  error_cevtg ~ cauchy(0, 2.5);


  // Calculate the mean and sd of the 3 curves.
  for (m in 1:M) {
    gas_mean[m] = gas_curve(k[m], beta0, beta);
    gas_sd[m]   = error_gas;
  }
  for (m in 1:(M - 1)) {
    cevgm_mean[m] = log(beta5);
    cevtg_mean[m] = log(beta3) + log(1 - exp(-beta4)) - (beta4 * (k[m]));

    cevgm_sd[m] = error_cevgm;
    cevtg_sd[m] = error_cevtg;
  }

  // Define the probability models for the 3 curves.
  gas      ~ lognormal(log(gas_mean),   gas_sd);
  cevgm_tf ~    normal(  cevgm_mean , cevgm_sd);
  cevtg_tf ~    normal(  cevtg_mean , cevtg_sd);
}
